@@ -355,4 +355,57 @@ mod tests {
         harness.ghost_actor_shutdown().await?;
         Ok(())
     }
+
+    /// Test that we can publish agent info.
+    #[tokio::test(flavor = "multi_thread")]
+    #[ignore = "Need a better way then waiting 6 minutes to test this"]
+    async fn test_publish_agent_info() {
+        observability::test_run().ok();
+
+        let (harness, _evt) = spawn_test_harness_mem().await.unwrap();
+
+        harness.add_space().await.unwrap();
+
+        // - Add the first agent
+        let (a1, _) = harness.add_publish_only_agent("one".into()).await.unwrap();
+
+        let peer_data = harness
+            .dump_local_peer_data(dbg!(a1.clone()))
+            .await
+            .unwrap();
+        let a1_peer_info = peer_data[&a1].clone();
+        // - Add the second agent
+        let (a2, _) = harness.add_publish_only_agent("two".into()).await.unwrap();
+
+        // - Add the second agent
+        let (a3, _) = harness
+            .add_publish_only_agent("three".into())
+            .await
+            .unwrap();
+
+        harness
+            .inject_peer_info(a3.clone(), a1_peer_info.clone())
+            .await
+            .unwrap();
+
+        // There's no way to trigger a publishing of peer data without waiting
+        // for > five minutes.
+        tokio::time::sleep(std::time::Duration::from_secs(6 * 60)).await;
+
+        let a1_peers = harness.dump_local_peer_data(a1.clone()).await.unwrap();
+        let a2_peers = harness.dump_local_peer_data(a2.clone()).await.unwrap();
+        let a3_peers = harness.dump_local_peer_data(a3.clone()).await.unwrap();
+
+        // a1 and a2 have each others peer info.
+        assert!(a1_peers.get(&a3).is_some());
+        assert!(a3_peers.get(&a1).is_some());
+
+        // a2 doesn't have anyone's info and no one has a2's info.
+        assert!(a1_peers.get(&a2).is_none());
+        assert!(a3_peers.get(&a2).is_none());
+        assert!(a2_peers.get(&a1).is_none());
+        assert!(a2_peers.get(&a3).is_none());
+
+        harness.ghost_actor_shutdown().await.unwrap();
+    }
 }
