@@ -3,7 +3,7 @@
 use crate::types::agent_store::AgentInfoSigned;
 use kitsune_p2p_timestamp::Timestamp;
 use kitsune_p2p_types::dht_arc::{DhtArcSet, DhtLocation};
-use std::{collections::HashSet, sync::Arc, time::SystemTime};
+use std::{collections::HashSet, sync::Arc};
 
 /// Gather a list of op-hashes from our implementor that meet criteria.
 /// Also get the start and end times for ops within a time window
@@ -126,77 +126,9 @@ impl QueryAgentsEvt {
     }
 }
 
-/// A single datum of metric info about an Agent, to be recorded by the client.
-#[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord, derive_more::Display)]
-pub enum MetricKind {
-    /// Our fast gossip loop synced this node up to this timestamp.
-    /// The next quick loop can sync from this timestamp forward.
-    QuickGossip,
-
-    /// The last time a full slow gossip loop completed was at this timestamp.
-    /// If that is too recent, we won't run another slow loop.
-    SlowGossip,
-
-    /// The last time we got a connection/timeout error with this node,
-    /// ignoring inactivity timeouts.
-    /// Lets us skip recently unreachable nodes in gossip loops.
-    ConnectError,
-}
-
-/// A single row in the metrics database
-#[derive(Clone, Debug, PartialEq, Eq)]
-pub struct MetricDatum {
-    /// The agent this event is about
-    pub agent: KAgent,
-    /// The kind of event
-    pub kind: MetricKind,
-    /// The time at which this occurred
-    pub timestamp: SystemTime,
-}
-
-/// The ordering is defined as such to facilitate in-memory metric store
-/// implementations such that the earliest and latest metrics can be easily obtained.
-impl PartialOrd for MetricDatum {
-    fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
-        match self.timestamp.cmp(&other.timestamp) {
-            std::cmp::Ordering::Equal => Some(self.agent.cmp(&other.agent)),
-            o => Some(o),
-        }
-    }
-}
-
-impl Ord for MetricDatum {
-    fn cmp(&self, other: &Self) -> std::cmp::Ordering {
-        self.partial_cmp(other).unwrap()
-    }
-}
-
-/// Different kinds of queries about metric data
-#[derive(Clone, Debug, PartialEq, Eq)]
-pub enum MetricQuery {
-    /// Filters for the "last sync" query.
-    LastSync {
-        /// The agent to query by
-        agent: KAgent,
-    },
-    /// Filters for the "oldest agent" query.
-    Oldest {
-        /// Agents whose last connection error is earlier than this time will be filtered out.
-        last_connect_error_threshold: std::time::SystemTime,
-    },
-}
-
-/// Corresponding response to `MetricQuery`
-#[derive(Clone, Debug, PartialEq, Eq)]
-pub enum MetricQueryAnswer {
-    /// The last sync time for all agents.
-    LastSync(Option<std::time::SystemTime>),
-    /// The agent with the oldest last-connection time which satisfies the query.
-    Oldest(Option<KAgent>),
-}
-
 /// An exclusive range of timestamps, measured in microseconds
 pub type TimeWindow = std::ops::Range<Timestamp>;
+
 /// An inclusive range of timestamps, measured in microseconds
 pub type TimeWindowInclusive = std::ops::RangeInclusive<Timestamp>;
 
@@ -204,10 +136,108 @@ pub type TimeWindowInclusive = std::ops::RangeInclusive<Timestamp>;
 pub fn full_time_window() -> TimeWindow {
     Timestamp::MIN..Timestamp::MAX
 }
+
 /// A time window which inclusively covers all of recordable time
 pub fn full_time_window_inclusive() -> TimeWindowInclusive {
     Timestamp::MIN..=Timestamp::MAX
 }
+
+const METRIC_KIND_UNKNOWN: &str = "Unknown";
+const METRIC_KIND_REACHABILITY_QUOTIENT: &str = "ReachabilityQuotient";
+const METRIC_KIND_LATENCY_MICROS: &str = "LatencyMicros";
+const METRIC_KIND_AGG_EXTRAP_COV: &str = "AggExtrapCov";
+
+/// The type of metric recorded
+pub enum MetricRecordKind {
+    /// Failure to parse metric kind
+    Unknown,
+
+    /// ReachabilityQuotient metric kind
+    ReachabilityQuotient,
+
+    /// LatencyMicros metric kind
+    LatencyMicros,
+
+    /// AggExtrapCov metric kind
+    AggExtrapCov,
+}
+
+impl MetricRecordKind {
+    /// database format of this kind variant
+    pub fn to_db(&self) -> &'static str {
+        use MetricRecordKind::*;
+        match self {
+            Unknown => METRIC_KIND_UNKNOWN,
+            ReachabilityQuotient => METRIC_KIND_REACHABILITY_QUOTIENT,
+            LatencyMicros => METRIC_KIND_LATENCY_MICROS,
+            AggExtrapCov => METRIC_KIND_AGG_EXTRAP_COV,
+        }
+    }
+
+    /// parse a database kind into a rust enum variant
+    pub fn from_db(input: &str) -> Self {
+        use MetricRecordKind::*;
+        if input == METRIC_KIND_REACHABILITY_QUOTIENT {
+            ReachabilityQuotient
+        } else if input == METRIC_KIND_LATENCY_MICROS {
+            LatencyMicros
+        } else if input == METRIC_KIND_AGG_EXTRAP_COV {
+            AggExtrapCov
+        } else {
+            Unknown
+        }
+    }
+}
+
+/// An individual metric record
+pub struct MetricRecord {
+    /// kind of this record
+    pub kind: MetricRecordKind,
+
+    /// agent associated with this metric (if applicable)
+    pub agent: Option<Arc<super::KitsuneAgent>>,
+
+    /// timestamp this metric was recorded at
+    pub recorded_at_utc: Timestamp,
+
+    /// timestamp this metric will expire and be available for pruning
+    pub expires_at_utc: Timestamp,
+
+    /// additional data associated with this metric
+    pub data: serde_json::Value,
+}
+
+/// Generic Kitsune Request of the implementor
+/// This enum may be easier to add variants to for future updates,
+/// rather than adding a full new top-level event message type.
+pub enum KGenReq {
+    /// Extrapolated Peer Coverage
+    PeerExtrapCov {
+        /// The space to extrapolate coverage
+        space: Arc<super::KitsuneSpace>,
+
+        /// Storage arcs of joined agents
+        dht_arc_set: DhtArcSet,
+    },
+
+    /// Record a set of metric records
+    RecordMetrics {
+        /// The space to associate the records with
+        space: Arc<super::KitsuneSpace>,
+
+        /// The records to record
+        records: Vec<MetricRecord>,
+    },
+}
+
+/// Generic Kitsune Respons from the imlementor
+pub enum KGenRes {
+    /// Extrapolated Peer Coverage
+    PeerExtrapCov(Vec<f64>),
+    /// Record a set of metric records
+    RecordMetrics(()),
+}
+
 type KSpace = Arc<super::KitsuneSpace>;
 type KAgent = Arc<super::KitsuneAgent>;
 type KOpHash = Arc<super::KitsuneOpHash>;
@@ -218,6 +248,9 @@ ghost_actor::ghost_chan! {
     /// The KitsuneP2pEvent stream allows handling events generated from the
     /// KitsuneP2p actor.
     pub chan KitsuneP2pEvent<super::KitsuneP2pError> {
+        /// Generic Kitsune Request of the implementor
+        fn k_gen_req(arg: KGenReq) -> KGenRes;
+
         /// We need to store signed agent info.
         fn put_agent_info_signed(input: PutAgentInfoSignedEvt) -> ();
 
@@ -230,12 +263,6 @@ ghost_actor::ghost_chan! {
         /// Query the peer density of a space for a given [`DhtArc`].
         fn query_peer_density(space: KSpace, dht_arc: kitsune_p2p_types::dht_arc::DhtArc) -> kitsune_p2p_types::dht_arc::PeerDensity;
 
-        /// Record a metric datum about an agent.
-        fn put_metric_datum(datum: MetricDatum) -> ();
-
-        /// Ask for metric data.
-        fn query_metrics(query: MetricQuery) -> MetricQueryAnswer;
-
         /// We are receiving a request from a remote node.
         fn call(space: KSpace, to_agent: KAgent, payload: Payload) -> Vec<u8>;
 
